@@ -36,16 +36,18 @@ from sqlalchemy import Integer
 from sqlalchemy import Unicode
 from sqlalchemy.event import listen
 from sqlalchemy.orm import Session
+from sqlalchemy.schema import Sequence
 
 from . exc import UninitializedError
 
 log = logging.getLogger(__name__)
-__all__ = [ 'FileSystemEntity' ]
+__all__ = ['FileSystemEntity']
 
 
 class FileSystemEntity(object):
 
-    id = Column(Integer, primary_key=True, autoincrement=True)
+    id_seq = Sequence("pufferfish_id_seq")
+    id = Column(Integer, id_seq, primary_key=True)
     content_type = Column(Unicode(128))
     name = Column(Unicode(128), nullable=False)
     size = Column(Integer)
@@ -83,7 +85,8 @@ class FileSystemEntity(object):
             To keep files in sync with database, it is needed to:
             - remove new added files as respective objects in database
               don't exist
-            - restore original files that have been updated during the transaction
+            - restore original files that have been updated during
+               the transaction
         """
         try:
             cls._mutex.acquire()
@@ -94,28 +97,28 @@ class FileSystemEntity(object):
             # delete new files as we are rolling back
             for obj in state["added"].values():
                 if obj.session is not session:
-                    log.debug("Skipping obj %s as it is attached to another "+\
-                              "session %s", obj, session)
+                    log.debug("Skipping obj %s as it is attached "
+                              "to another session %s", obj, session)
                     continue
 
                 if os.path.isdir(obj.dir):
-                    log.debug("%s is in the added state but transaction has been rolled back. Removing",
-                                 obj.dir)
+                    log.debug("%s is in the added state but transaction has "
+                              "been rolled back. Removing", obj.dir)
                     try:
                         shutil.rmtree(obj.dir)
                     except Exception:
                         cls._state['failed'].append(obj)
-                        log.exception("Error in handling rollback for %s" % obj)
+                        log.exception("Error in rollback for %s" % obj)
                 del cls._state[cls.__name__]['added'][obj.id]
 
             # restore original version of updated files
             for obj in state['updated'].values():
                 if obj.session is not session:
-                    log.debug("Skipping obj %s as it is attached to another "+\
-                              "session %s", obj, session)
+                    log.debug("Skipping obj %s as it is attached "
+                              "to another session %s", obj, session)
                     continue
 
-                log.debug("%s is in the updated state but transaction has " +\
+                log.debug("%s is in the updated state but transaction has "
                           "been rolled back. Moving back old file from %s",
                           obj.name, obj.original_dir)
                 try:
@@ -159,8 +162,9 @@ class FileSystemEntity(object):
             # transaction has been committed, remove files from disc
             for obj in state["deleted"].values():
                 if obj.session is not session:
-                    log.debug("Skipping obj %s as it is attached to another "+\
-                              "session %s", obj, session)
+                    log.debug("Skipping obj %s as it is"
+                              "attached to another session %s",
+                              obj, session)
                     continue
 
                 log.debug("%s is in the removed state and transaction has " +\
@@ -211,8 +215,6 @@ class FileSystemEntity(object):
             url_prefix = '/' + url_prefix
         cls.url_prefix = url_prefix
 
-
-
     @classmethod
     def initialize(cls, base, private="", url_prefix=None):
         log.info("Initializing %s", cls.__name__)
@@ -225,7 +227,6 @@ class FileSystemEntity(object):
         listen(Session, 'after_commit', cls.after_commit)
         listen(Session, 'after_attach', cls.after_attach)
 
-
     @classmethod
     def after_delete(cls, mapper, connection, target):
         # add the path to the list of those that will be
@@ -312,7 +313,6 @@ class FileSystemEntity(object):
         except:
             pass
 
-
     def __init__(self, *args, **kwargs):
         kwargs.pop("source", None)
         kwargs.pop("session", None)
@@ -334,11 +334,11 @@ class FileSystemEntity(object):
     def create_new(cls, self, args, kwargs):
         log.debug("Running post-constructor on %s, (args:%s, kwargs:%s)",
                   cls.__name__, args, kwargs)
-        if not self.id is None:
-            return
 
         if "source" not in kwargs:
-            raise TypeError("takes at least source argument")
+            if not self.id:
+                raise TypeError('Source parameter is required')
+            return
 
         if "name" not in kwargs:
             temp_name = True
@@ -351,9 +351,13 @@ class FileSystemEntity(object):
         session = kwargs.pop('session')
         object.__setattr__(self, "name", kwargs['name'])
 
-        # flush the session, so that the DBMS assigns us an unique ID
+        if 'id' in kwargs:
+            object.__setattr__(self, 'id', kwargs['id'])
+
         session.add(self)
-        session.flush()
+        if not self.id:
+            # flush the session, so that the DBMS assigns us an unique ID
+            session.flush()
 
         self.content_type = magic.from_buffer(source.read(1024), mime=True)
         log.debug("content_type: %s", self.content_type)
@@ -372,9 +376,10 @@ class FileSystemEntity(object):
             confl = "%s%s" % (self.dir, "_conflict")
             while os.path.isdir(confl):
                 confl = "%s%s" % (confl, "_")
-            log.error("%s already exists, while it should not as the primary key %d as " +\
-                      "been just generated. This is problably a bug. "+ \
-                      "Moving %s to %s", self.dir, self.id, self.dir, confl)
+            log.error("%s already exists, while it should not as the "
+                      "primary key %d has been just generated. This is "
+                      "problably a bug. Moving %s to %s",
+                      self.dir, self.id, self.dir, confl)
 
             shutil.move(self.dir, confl)
             os.makedirs(self.dir)
@@ -387,8 +392,8 @@ class FileSystemEntity(object):
         del temp_name
 
     def clean_name(self, value):
-         value = value.replace(" ", "_")
-         return re.sub('[^\w\._]+', '', value)
+        value = value.replace(" ", "_")
+        return re.sub('[^\w\._]+', '', value)
 
     def __protect_original(self):
         # if file already exists, rename it first
@@ -448,7 +453,6 @@ class FileSystemEntity(object):
 
         raise AttributeError(attr)
 
-
     def __setattr__(self, attr, value):
         if attr == "name":
             if value == self.name:
@@ -458,7 +462,7 @@ class FileSystemEntity(object):
             ext = os.path.splitext(value)[1]
             if ext == "" and self.content_type is not None:
                 ext = mimetypes.guess_extension(self.content_type)
-                if ext == ".jpe": # pragma: nocover
+                if ext == ".jpe":  # pragma: nocover
                     ext = ".jpg"
                 value = "%s%s" % (self.id, ext.lower())
             self.__update_file_name(value)
@@ -469,4 +473,3 @@ class FileSystemEntity(object):
     def to_dict(self):
         return dict(id=self.id, name=self.name, url=self.url, path=self.path,
                     size=self.size, content_type=self.content_type)
-
